unit Field2;

uses GraphABC;

const
  StandardDelay=30;
  maxDim=40;

procedure Pause;
begin
  Sleep(StandardDelay);
end;

type
  Field=class
    DimX,DimY,sz,x0,y0: integer;

    procedure Draw;
    var cx,cy,w,h: integer;
    begin
      SetPenColor(clBlack);
      w:=sz*DimX;
      h:=sz*DimY;
      for cy:=0 to DimY do
        line(x0,y0+cy*sz,x0+w+1,y0+cy*sz);
      for cx:=0 to DimX do
        line(x0+cx*sz,y0,x0+cx*sz,y0+h);
    end;
    
    constructor Create(dx,dy,ssz,x,y: integer);
    begin
      DimX:=dx; DimY:=dy; sz:=ssz; x0:=x; y0:=y;
    end;

    procedure DrawCircle(x,y,zaz,color: integer);
    begin
      SetBrushColor(color);
      Ellipse(x0+(x-1)*sz+zaz,y0+(y-1)*sz+zaz,x0+x*sz-zaz+1,y0+y*sz-zaz+1);
    end;

    procedure DrawRect(x,y,zaz,color: integer);
    begin
      SetBrushColor(color);
      Rectangle(x0+(x-1)*sz+zaz,y0+(y-1)*sz+zaz,x0+x*sz-zaz+1,y0+y*sz-zaz+1);
    end;

    procedure ClearCell(x,y: integer);
    begin
      SetBrushColor(clWhite);
      FillRect(x0+(x-1)*sz+1,y0+(y-1)*sz+1,x0+x*sz-1+1,y0+y*sz-1+1);
    end;

    procedure SetDim(dx,dy,ssz: integer);
    begin
      DimX:=dx; DimY:=dy; sz:=ssz;
    end;
    
    function CellIsFree(x,y: integer): boolean;
    begin
      Result:=True; //       
    end;

    function CellIsOccupied(x,y: integer): boolean;
    begin
      Result:=not CellIsFree(x,y);
    end;

    function CellInField(x,y: integer): boolean;
    begin
      Result:=(x>=1) and (y>=1) and (x<=DimX) and (y<=DimY)
    end;

  end;
  
  Ball=class
    x,y,color: integer;
    F: Field;

    constructor Create(xx,yy,cc: integer; FF: Field);
    begin
      x:=xx; y:=yy;
      color:=cc;
      F:=FF;
    end;

    constructor Create0(cc: integer; FF: Field);
    begin
      x:=1; y:=1;
      color:=cc;
      F:=FF;
    end;

    procedure Draw;
    begin
      F.DrawCircle(x,y,3,color);
    end;

    procedure Hide;
    begin
      F.ClearCell(x,y);
    end;

    procedure SetCoords(xx,yy: integer);
    begin
      x:=xx;
      y:=yy;
    end;

    procedure MoveTo(xx,yy: integer);
    begin
      Hide;
      SetCoords(xx,yy);
      Draw;
    end;

    procedure MoveOn(dx,dy: integer);
    begin
      MoveTo(x+dx,y+dy)
    end;
    
    function CellIsFree(dx,dy: integer): boolean;
    begin
      Result:=F.CellIsFree(x+dx,y+dy);
    end;

    function CellIsOccupied(dx,dy: integer): boolean;
    begin
      Result:=F.CellIsOccupied(x+dx,y+dy);
    end;
    
    procedure Reset;
    begin
    end;

    procedure Go;
    begin
    end;
  end;
  
  FieldWithBalls=class(Field)
    A: array [1..MaxDim,1..MaxDim] of Ball;
    
    procedure ClearBallArray;
    var ix,iy: integer;
    begin
      for ix:=1 to DimX do
      for iy:=1 to DimY do
        A[ix,iy]:=nil;
    end;

    constructor Create(dx,dy,ssz,x,y: integer);
    begin
      inherited Create(dx,dy,ssz,x,y);
      ClearBallArray;
    end;
    
    procedure DrawObjects;
    var ix,iy: integer;
    begin
      for ix:=1 to DimX do
      for iy:=1 to DimY do
        if A[ix,iy]<>nil then A[ix,iy].Draw;
    end;

    procedure Draw;
    begin
      inherited Draw;
      DrawObjects;
    end;
    
    procedure AddBall(B: Ball; x,y: integer);
    begin
      if not CellIsFree(x,y) then Exit;
      A[x,y]:=B;
      B.x:=x;
      B.y:=y;
      B.Draw;
    end;

    function CellIsFree(x,y: integer): boolean;
    begin
      Result:=CellInField(x,y) and (A[x,y]=nil)
    end;
    
    procedure MoveObject(x,y,x1,y1: integer);
    begin
      if CellIsFree(x,y) then exit;
      if not CellIsFree(x1,y1) then exit;
      A[x,y].Hide;
      A[x1,y1]:=A[x,y];
      A[x,y]:=nil;
      A[x1,y1].SetCoords(x1,y1);
      A[x1,y1].Draw;
    end;
    
    procedure DeleteObject(x,y: integer);
    begin
      if A[x,y]=nil then exit;
      A[x,y].Destroy;
      A[x,y]:=nil;
      ClearCell(x,y);
    end;

    procedure Go;
    var ix,iy:integer;
    begin
      for ix:=1 to DimX do
      for iy:=1 to DimY do
        if A[ix,iy]<>nil then
          A[ix,iy].Reset;
      for ix:=1 to DimX do
      for iy:=1 to DimY do
        if A[ix,iy]<>nil then
          A[ix,iy].Go;
    end;
  end;
  
  MovingBall=class(Ball)
    dx,dy: integer;
    goflag: boolean;
    constructor Create0(ddx,ddy,c: integer; F: Field);
    begin
      inherited Create0(c,F);
      goflag:=false;
      dx:=ddx; dy:=ddy;
    end;
    
    procedure Reset;
    begin
      goflag:=false;
    end;
    
    procedure MoveTo(xx,yy: integer);
    //  -    .      
    //         
    var FF: FieldWithBalls;
    begin
      FF:=FieldWithBalls(F);
      FF.MoveObject(x,y,xx,yy)
    end;

    procedure Go;
    begin
      if goflag=true then exit;
      if (CellIsOccupied(dx,0) and CellIsOccupied(0,dy)) or (CellIsFree(dx,0) and CellIsFree(0,dy) and CellIsOccupied(dx,dy)) then
      begin
        dx:=-dx;
        dy:=-dy;
      end
      else if (dx<>0) and CellIsOccupied(dx,0) then
        dx:=-dx
      else if (dy<>0) and CellIsOccupied(0,dy) then
        dy:=-dy;
      MoveOn(dx,dy);
      goflag:=True;
    end;
  end;
  
  BrownBall=class(MovingBall)
    constructor Create0(c: integer; F: Field);
    begin
      inherited Create0(0,0,c,F);
    end;

    procedure Go;
    begin
      if goflag=true then exit;
      dx:=Random(3)-1;
      dy:=Random(3)-1;
      MoveOn(dx,dy);
      goflag:=True;
    end;
    procedure Draw;
    begin
      F.DrawCircle(x,y,4,color);
    end;
  end;

  BrownEatingBall=class(BrownBall)
    procedure Go;
    var FF: FieldWithBalls;
    begin
      FF:=FieldWithBalls(F);
      if goflag=true then exit;
      repeat
        dx:=Random(3)-1;
        dy:=Random(3)-1;
      until (dx<>0) or (dy<>0);
      if FF.CellInField(x+dx,y+dy) and CellIsOccupied(dx,dy) then
        FF.DeleteObject(x+dx,y+dy);
      MoveOn(dx,dy);
      goflag:=True;
    end;
    procedure Draw;
    begin
      F.DrawRect(x,y,3,color);
    end;
  end;

  Pulya=class(MovingBall)
    procedure Go;
    var FF: FieldWithBalls;
    begin
      FF:=FieldWithBalls(F);
      if goflag=true then exit;
      if FF.CellInField(x+dx,y+dy) and CellIsOccupied(dx,dy) then
      begin
        FF.DeleteObject(x+dx,y+dy);
        FF.DeleteObject(x,y);
        exit;
      end;
      if not FF.CellInField(x+dx,y+dy) then
      begin
        FF.DeleteObject(x,y);
        exit;
      end;
      MoveOn(dx,dy);
      goflag:=True;
    end;
    procedure Draw;
    begin
      F.DrawCircle(x,y,6,color);
    end;
  end;

  ShootingBall=class(BrownBall)
    period: integer;
    procedure Go;
    var FF: FieldWithBalls;
    begin
      FF:=FieldWithBalls(F);
      if goflag=true then exit;
      repeat
        dx:=Random(3)-1;
        dy:=Random(3)-1;
      until (dx<>0) or (dy<>0);
      Inc(period);
      if period>10 then
      begin
        period:=0;
        if CellIsFree(1,0) then FF.AddBall(Pulya.Create0(1,0,clBlack,F),x+1,y);
        if CellIsFree(-1,0) then FF.AddBall(Pulya.Create0(-1,0,clBlack,F),x-1,y);
        if CellIsFree(0,1) then FF.AddBall(Pulya.Create0(0,1,clBlack,F),x,y+1);
        if CellIsFree(0,-1) then FF.AddBall(Pulya.Create0(0,-1,clBlack,F),x,y-1);
      end else
        MoveOn(dx,dy);
      goflag:=True;
    end;
    procedure Draw;
    begin
      F.DrawCircle(x,y,0,color);
    end;
  end;
  
  
end.
